""" Parsergroups for daily.
"""

import argparse


def create_filter_opts():
    parser = argparse.ArgumentParser(add_help=False)
    group = parser.add_argument_group(
        'filter options',
        'Filter for entries on the specified criteria.')
    group.add_argument(
        '--after', metavar='DATE',
        help='Filter for transactions after this date.')
    group.add_argument(
        '--before', metavar='DATE',
        help='Filter for entries after this date.')
    group.add_argument(
        '-d', '--date',
        help='The entry on this date.')
    group.add_argument(
        '--tags',
        help='Filter for entries with any of the mentioned tags.',
        metavar='TAG[,TAG... ,TAG]', default='')

    return parser



from daily_cli.parsergroups import create_filter_opts

filter_opts = create_filter_opts()


def add_subparser(subparsers):
    sp = subparsers.add_parser(
        'show',
        help='Display journal entries.',
        parents=[filter_opts])

    sp.add_argument(
        'headings', metavar='HEADING', nargs='*',
        help='Only show entries made under the specified headings.')



import importlib
import inspect
import os


_pkg_root = os.path.dirname(os.path.abspath(__file__))


def get_subcommands(pkg_root):
    """ List available subcommands within a module.

    Args:
        pkg_root: a path to the root of the subpackage to be searched.

    Returns:
        A list of strings representing the names of all subpackages
        present in the given directory.
    """
    return [dname for dname in sorted(os.listdir(pkg_root)) if
            os.path.isdir(os.path.join(pkg_root, dname))
            and '__pycache__' not in dname]


def build_out_subparsers(subparser_hook):
    """ Adds subcommand subparsers to a parent parser.

    This command will dynamically search all subpackages of `daily.cli`,
    import the `parser` submodule of every subpackage, and execute a
    function named `add_*_subparser` on a `subparsers` object returned
    by `ArgumentParser.add_subparsers()`. This will have the effect of
    creating all subparsers for all available subcommands.

    Args:
        subparser_hook: an object returned by
            ArgumentParser.add_subparsers().

    Returns:
        None
    """
    parser_modules = [importlib.import_module(f'daily_cli.cli.{x}.parser') for x in get_subcommands(_pkg_root)]
    parser_builders = []
    for module in parser_modules:
        fns = [item for name, item in inspect.getmembers(module)
               if inspect.isfunction(item) and name == 'add_subparser']
        if len(fns) != 1:
            raise RuntimeError(f'Too many functions in {module.__name__}')
        else:
            parser_builders.extend(fns)

    for builder in parser_builders:
        builder(subparser_hook)



import os
import subprocess as sp
from setuptools import setup, find_packages
from os import path


here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), 'r', encoding='utf-8') as f:
    long_description = f.read()

# Find man pages.
manfiles = []
for r, d, f in os.walk(path.join(here, 'docs', 'man')):
    manfiles = [path.join(r, f) for f in f if f.endswith('.1.gz')]
    break

install_requires = []
if 'DEBBUILD' not in os.environ:
    install_requires = [
        'argcomplete',
        'libzet>=2.1.2',
    ]

setup(
    name='daily-cli',
    version='1.0.1',
    description='The command-line notebook for daily entries.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='',
    author='Logan Reece',
    author_email='onereddime@protonmail.com',
    packages=find_packages(exclude=['tests']),
    license='GPLv2',
    python_requires='>=3, <4',
    install_requires=install_requires,
    data_files=[
        ('etc/bash_completion.d', ['etc/daily_completion.sh']),
        ('usr/share/man/man1', manfiles),
    ],
    entry_points={
        'console_scripts': [
            'daily=daily_cli.main:main',
        ]
    },
)



from daily_cli.parsergroups import create_filter_opts


filter_opts = create_filter_opts()


def add_subparser(subparsers):
    sp = subparsers.add_parser(
        'add', help='Add a new entry or modify an existing one.',
        parents=[filter_opts])

    sp.add_argument(
        '--no-edit', action='store_true',
        help=('Create a new empty entry for the given date. '
              'Only compatible with -d option.'))

    sp.add_argument(
        'headings', metavar='HEADING',
        help='Add (or modify) specific headings for an entry.', nargs='*')



import os
from datetime import datetime
from glob import glob

from libzet import load_zettels
from superdate import parse_date


def get_title_from_date(date, format_='%Y-%m-%d, %a'):
    """ Generate a more human-readable title.

    Returns:
        A string showing the date in Y-m-d format and the weekday.

    Raises:
        ValueError if date could not be parsed.
    """
    return parse_date(date).strftime(format_)


def entry_filter(entry, args):
    """ Returns True if the entry satisfies the filters in args.
    """
    is_after = args.after and entry.title >= args.after
    is_before = args.before and entry.title <= args.before
    is_date = args.date and entry.title == args.date

    if not any([args.after, args.before, args.date]):
        is_after = is_before = is_date = True
    else:
        if args.before and not args.after:
            is_after = True
        if args.after and not args.before:
            is_before = True

    in_date = (is_after and is_before) or is_date
    in_tags = not args.tags or any([x for x in args.tags if x in entry.attrs['tags']])

    return in_date and in_tags


def load_entries(args):
    """ Load entries from args.

    Args:
        The argparse instance used by daily commands.

    Returns:
        A list of the loaded zettels.

    Raises:
        FileNotFoundError if the journal file doesn't exist.
        PermissionError if the journal file couldn't be read.

        ValueError if an entry in the journal contains an invalid entry.
        The message will indicate which entry and the error.
    """
    if args.before:
        args.before = get_title_from_date(args.before)
    if args.after:
        args.after = get_title_from_date(args.after)
    if args.date:
        args.date = get_title_from_date(args.date)

    if args.before and args.after and args.before < args.after:
        print('ERROR: The "before" date cannot be before the "after" date.')
        sys.exit(1)

    args.tags = [x for x in args.tags.split(',') if x]

    match_ = f'[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9].{args.entry_format}'
    paths = glob(f'{args.journal}/{match_}')

    zettels = load_zettels(paths, args.entry_format)
    zettels = [z for z in zettels if entry_filter(z, args)]

    return zettels
